package cc.fascinated.wildaddons.statistic;

import cc.fascinated.wildaddons.Main;
import cc.fascinated.wildaddons.utils.Config;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import org.spongepowered.configurate.ConfigurateException;
import org.spongepowered.configurate.ConfigurationNode;
import org.spongepowered.configurate.serialize.SerializationException;

import java.util.HashMap;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

@Getter @RequiredArgsConstructor
public enum Statistic {

    PLAYERS_WELCOMED("Players Welcomed", "players-welcomed"),
    FILTERED_MESSAGES("Filtered Messages","filtered-messages");

    /**
     * The in memory storage for all statistics
     */
    @Getter private static final HashMap<Statistic, Integer> statistics = new HashMap<>();

    /**
     * If the values is dirty, then we can save to disk
     */
    private static boolean dirty;

    /**
     * The display name for the statistic
     */
    private final String name;

    /**
     * The key used in the storage
     */
    private final String key;

    /**
     * Gets the value of this statistic.
     *
     * @return the value
     */
    public int get() {
        return statistics.computeIfAbsent(this, (e) -> 0);
    }

    /**
     * Increment this statistic.
     */
    public void increment() {
        increment(1);
    }

    /**
     * Increment this statistic.
     *
     * @param amount the amount to increment by
     */
    public void increment(int amount) {
        int current = statistics.computeIfAbsent(this, (e) -> 0);
        current += amount;
        statistics.put(this, current);
        dirty = true;
    }

    public static void initSaver() {
        Config statisticsConfig = Main.getStatisticsConfig();
        new Timer().scheduleAtFixedRate(new TimerTask() {
            @Override
            public void run() {
                if (!dirty) {
                    return;
                }
                for (Statistic value : values()) {
                    try {
                        statisticsConfig.getNode().node(value.key).set(value.get());
                    } catch (SerializationException e) {
                        throw new RuntimeException(e);
                    }
                }
                try {
                    statisticsConfig.save();
                } catch (ConfigurateException e) {
                    throw new RuntimeException(e);
                }

                dirty = false;
                Main.LOGGER.info("Saved statistics.");
            }
        }, 60_000L, 60_000L);
    }

    /**
     * Loads statistics from disk
     *
     * @param node the configuration node to load from
     */
    public static void loadStatistics(ConfigurationNode node) {
        for (Map.Entry<Object, ? extends ConfigurationNode> entry : node.childrenMap().entrySet()) {
            ConfigurationNode data = entry.getValue();
            String key = (String) data.key();
            int value = data.getInt();

            statistics.put(Statistic.getStatisticFromKey(key), value);
        }
        Main.LOGGER.info("Loaded statistics.");
    }

    /**
     * Gets a statistic from the key
     *
     * @param key the key to check against
     * @return the statistic if found, otherwise null
     */
    public static Statistic getStatisticFromKey(String key) {
        for (Statistic value : values()) {
            if (value.key.equalsIgnoreCase(key)) {
                return value;
            }
        }
        return null;
    }
}
